<?php

namespace PHPMaker2021\educlaims;

use Slim\Views\PhpRenderer;
use Slim\Csrf\Guard;
use Psr\Container\ContainerInterface;
use Monolog\Logger;
use Monolog\Handler\RotatingFileHandler;
use Doctrine\DBAL\Logging\LoggerChain;
use Doctrine\DBAL\Logging\DebugStack;

return [
    "cache" => function (ContainerInterface $c) {
        return new \Slim\HttpCache\CacheProvider();
    },
    "view" => function (ContainerInterface $c) {
        return new PhpRenderer("views/");
    },
    "flash" => function (ContainerInterface $c) {
        return new \Slim\Flash\Messages();
    },
    "audit" => function (ContainerInterface $c) {
        $logger = new Logger("audit"); // For audit trail
        $logger->pushHandler(new AuditTrailHandler("/var/www/logs/audit.log"));
        return $logger;
    },
    "log" => function (ContainerInterface $c) {
        global $RELATIVE_PATH;
        $logger = new Logger("log");
        $logger->pushHandler(new RotatingFileHandler($RELATIVE_PATH . "/var/www/logs/log.log"));
        return $logger;
    },
    "sqllogger" => function (ContainerInterface $c) {
        $loggers = [];
        if (Config("DEBUG")) {
            $loggers[] = $c->get("debugstack");
        }
        return (count($loggers) > 0) ? new LoggerChain($loggers) : null;
    },
    "csrf" => function (ContainerInterface $c) {
        global $ResponseFactory;
        return new Guard($ResponseFactory, Config("CSRF_PREFIX"));
    },
    "debugstack" => \DI\create(DebugStack::class),
    "debugsqllogger" => \DI\create(DebugSqlLogger::class),
    "security" => \DI\create(AdvancedSecurity::class),
    "profile" => \DI\create(UserProfile::class),
    "language" => \DI\create(Language::class),
    "timer" => \DI\create(Timer::class),
    "session" => \DI\create(HttpSession::class),

    // Tables
    "claims" => \DI\create(Claims::class),
    "audit" => \DI\create(Audit::class),
    "user" => \DI\create(User::class),
    "welcome" => \DI\create(Welcome::class),
    "userattrs" => \DI\create(Userattrs::class),
    "clm_categories" => \DI\create(ClmCategories::class),
    "v_usr" => \DI\create(VUsr::class),
    "v_clm" => \DI\create(VClm::class),
    "signature" => \DI\create(Signature::class),
    "signpage" => \DI\create(Signpage::class),
    "tosign" => \DI\create(Tosign::class),
    "codebook_items" => \DI\create(CodebookItems::class),
    "yourclaims" => \DI\create(Yourclaims::class),

    // User table
    "usertable" => \DI\get("user"),
];
